@@ -22,6 +22,7 @@ import java.io.File;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.net.URI;
+import java.net.URISyntaxException;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
@@ -55,21 +56,26 @@ import org.apache.commons.net.imap.IMAPReply;
  * <ul>
  * <li>BODY.PEEK[HEADER]</li>
  * <li>'BODY.PEEK[HEADER.FIELDS (SUBJECT)]'</li>
- * <li>ALL</li>
- * <li>ENVELOPE</li>
+ * <li>ALL - macro equivalent to '(FLAGS INTERNALDATE RFC822.SIZE ENVELOPE)'</li>
+ * <li>FAST - macro equivalent to '(FLAGS INTERNALDATE RFC822.SIZE)'</li>
+ * <li>FULL - macro equivalent to '(FLAGS INTERNALDATE RFC822.SIZE ENVELOPE BODY)'</li>
+ * <li>ENVELOPE X-GM-LABELS</li>
  * <li>'(INTERNALDATE BODY.PEEK[])' - this is the default</li>
  * </ul>
  * <p>
+ * Macro names cannot be combined with anything else; they must be used alone.<br>
+ * Note that using BODY will set the \Seen flag. This is why the default uses BODY.PEEK[].<br>
+ * The item name X-GM-LABELS is a Google Mail extension; it shows the labels for a message.<br>
  * For example:<br>
  * IMAPExportMbox imaps://username:password@imap.googlemail.com/messages_for_export exported.mbox 1:10,20<br>
- * IMAPExportMbox imaps://username:password@imap.googlemail.com/messages_for_export exported.mbox 3 ENVELOPE<br>
+ * IMAPExportMbox imaps://username:password@imap.googlemail.com/messages_for_export exported.mbox 3 ENVELOPE X-GM-LABELS<br>
  * <p>
- * Note that the sequence-set is passed unmodified to the FETCH command.
+ * The sequence-set is passed unmodified to the FETCH command.<br>
  * The item names are wrapped in parentheses if more than one is provided.
- * Otherwise, the parameter is assumed to be wrapped if necessary.
- * Parameters with spaces must be quoted otherwise the OS shell will normally treat them as separate parameters.
- * Also the listener that writes the mailbox only captures the multi-line responses.
- * It does not capture the output from ENVELOPE commands.
+ * Otherwise, the parameter is assumed to be wrapped if necessary.<br>
+ * Parameters with spaces must be quoted otherwise the OS shell will normally treat them as separate parameters.<br>
+ * Also the listener that writes the mailbox only captures the multi-line responses (e.g. ones that include BODY references).
+ * It does not capture the output from FETCH commands using item names such as ENVELOPE or FLAGS that return a single line response.
  */
 public final class IMAPExportMbox
 {
@@ -96,7 +102,7 @@ public final class IMAPExportMbox
     private static final int CONNECT_TIMEOUT = 10; // Seconds
     private static final int READ_TIMEOUT = 10;
 
-    public static void main(String[] args) throws IOException
+    public static void main(String[] args) throws IOException, URISyntaxException
     {
         int connect_timeout = CONNECT_TIMEOUT;
         int read_timeout = READ_TIMEOUT;
@@ -147,7 +153,19 @@ public final class IMAPExportMbox
             System.exit(1);
         }
 
-        final URI uri      = URI.create(args[argIdx++]);
+        final String uriString = args[argIdx++];
+        URI uri;
+        try {
+            uri      = URI.create(uriString);
+        } catch(IllegalArgumentException e) { // cannot parse the path as is; let's pull it apart and try again
+            Matcher m = Pattern.compile("(imaps?://[^/]+)(/.*)").matcher(uriString);
+            if (m.matches()) {
+                uri = URI.create(m.group(1)); // Just the scheme and auth parts
+                uri = new URI(uri.getScheme(), uri.getAuthority(), m.group(2), null, null);
+            } else {
+                throw e;
+            }
+        }
         final String file  = args[argIdx++];
         String sequenceSet = argCount > 2 ? args[argIdx++] : "1:*";
         final String itemNames;
@@ -187,8 +205,8 @@ public final class IMAPExportMbox
                 new BufferedWriter(new FileWriter(mbox, false)), eol, printHash, printMarker, checkSequence);
         } else {
             final File mbox = new File(file);
-            if (mbox.exists()) {
-                throw new IOException("mailbox file: " + mbox + " already exists!");
+            if (mbox.exists() && mbox.length() > 0) {
+                throw new IOException("mailbox file: " + mbox + " already exists and is non-empty!");
             }
             System.out.println("Creating file " + mbox);
             chunkListener = new MboxListener(new BufferedWriter(new FileWriter(mbox)), eol, printHash, printMarker, checkSequence);
