@@ -17,12 +17,8 @@
 
 package examples.mail;
 
-import java.io.BufferedReader;
-import java.io.Console;
 import java.io.IOException;
-import java.io.InputStreamReader;
 import java.net.URI;
-import java.util.Locale;
 
 import org.apache.commons.net.ProtocolCommandListener;
 import org.apache.commons.net.imap.IMAPClient;
@@ -57,37 +53,8 @@ class IMAPUtils {
 
         String username = userpass[0];
         String password = userpass[1];
-        /*
-         * If the initial password is:
-         * '*' - replace it with a line read from the system console
-         * '-' - replace it with next line from STDIN
-         * 'ABCD' - if the input is all upper case, use the field as an environment variable name
-         *
-         * Note: there are no guarantees that the password cannot be snooped.
-         *
-         * Even using the console may be subject to memory snooping,
-         * however it should be safer than the other methods.
-         *
-         * STDIN may require creating a temporary file which could be read by others
-         * Environment variables may be visible by using PS
-         */
-        if ("-".equals(password)) { // stdin
-            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
-            password = in.readLine();
-        } else if ("*".equals(password)) { // console
-            Console con = System.console(); // Java 1.6
-            if (con != null) {
-                char[] pwd = con.readPassword("Password for " + username + ": ");
-                password = new String(pwd);
-            } else {
-                throw new IOException("Cannot access Console");
-            }
-        } else if (password.equals(password.toUpperCase(Locale.ROOT))) { // environment variable name
-            final String tmp = System.getenv(password);
-            if (tmp != null) { // don't overwrite if variable does not exist (just in case password is all uppers)
-                password=tmp;
-            }
-        }
+        // prompt for the password if necessary
+        password = Utils.getPassword(username, password);
 
         final IMAPClient imap;
 
