@@ -45,7 +45,7 @@ import org.apache.commons.net.util.TrustManagerUtils;
  * Warning: the hostname is not verified against the certificate by default, use
  * {@link #setHostnameVerifier(HostnameVerifier)} or {@link #setEndpointCheckingEnabled(boolean)}
  * (on Java 1.7+) to enable verification. Verification is only performed on client mode connections.
- * @version $Id: FTPSClient.java 1697433 2015-08-24 15:11:40Z sebb $
+ * @version $Id: FTPSClient.java 1747829 2016-06-11 00:57:57Z sebb $
  * @since 2.0
  */
 public class FTPSClient extends FTPClient {
@@ -740,13 +740,16 @@ public class FTPSClient extends FTPClient {
      * Calls {@code setSocketFactory(null)} and {@code setServerSocketFactory(null)}
      * to reset the factories that may have been changed during the session,
      * e.g. by {@link #execPROT(String)}
-     * @exception IOException If an error occurs while disconnecting.
+     * @throws IOException If an error occurs while disconnecting.
      * @since 3.0
      */
     @Override
     public void disconnect() throws IOException
     {
         super.disconnect();
+        if (plainSocket != null) {
+            plainSocket.close();
+        }
         setSocketFactory(null);
         setServerSocketFactory(null);
     }
