@@ -121,14 +121,15 @@ public class DefaultFTPFileEntryParserFactory
 
         if (parser == null) { // Now try for aliases
             String ukey = key.toUpperCase(java.util.Locale.ENGLISH);
-            if (ukey.indexOf(FTPClientConfig.SYST_UNIX) >= 0)
-            {
-                parser = new UnixFTPEntryParser(config, false);
-            }
-            else if (ukey.indexOf(FTPClientConfig.SYST_UNIX_TRIM_LEADING) >= 0)
+            if (ukey.indexOf(FTPClientConfig.SYST_UNIX_TRIM_LEADING) >= 0)
             {
                 parser = new UnixFTPEntryParser(config, true);
             }
+            // must check this after SYST_UNIX_TRIM_LEADING as it is a substring of it
+            else if (ukey.indexOf(FTPClientConfig.SYST_UNIX) >= 0)
+            {
+                parser = new UnixFTPEntryParser(config, false);
+            }
             else if (ukey.indexOf(FTPClientConfig.SYST_VMS) >= 0)
             {
                 parser = new VMSVersioningFTPEntryParser(config);
@@ -192,7 +193,7 @@ public class DefaultFTPFileEntryParserFactory
      * used to configure the parser created
      *
      * @return the @link  FTPFileEntryParser FTPFileEntryParser} so created.
-     * @exception ParserInitializationException
+     * @throws ParserInitializationException
      *                   Thrown on any exception in instantiation
      * @throws NullPointerException if {@code config} is {@code null}
      * @since 1.4
@@ -239,11 +240,13 @@ public class DefaultFTPFileEntryParserFactory
         {
             return new NTFTPEntryParser(config);
         } else {
+            // clone the config as it may be changed by the parsers (NET-602)
+            final FTPClientConfig config2 =  (config != null) ? new FTPClientConfig(config) : null;
             return new CompositeFileEntryParser(new FTPFileEntryParser[]
                    {
                        new NTFTPEntryParser(config),
-                       new UnixFTPEntryParser(config,
-                               config != null && FTPClientConfig.SYST_UNIX_TRIM_LEADING.equals(config.getServerSystemKey()))
+                       new UnixFTPEntryParser(config2,
+                               config2 != null && FTPClientConfig.SYST_UNIX_TRIM_LEADING.equals(config2.getServerSystemKey()))
                    });
         }
     }
@@ -272,11 +275,13 @@ public class DefaultFTPFileEntryParserFactory
         {
             return new OS400FTPEntryParser(config);
         } else {
+            // clone the config as it may be changed by the parsers (NET-602)
+            final FTPClientConfig config2 =  (config != null) ? new FTPClientConfig(config) : null;
             return new CompositeFileEntryParser(new FTPFileEntryParser[]
                 {
                     new OS400FTPEntryParser(config),
-                    new UnixFTPEntryParser(config,
-                            config != null && FTPClientConfig.SYST_UNIX_TRIM_LEADING.equals(config.getServerSystemKey()))
+                    new UnixFTPEntryParser(config2,
+                            config2 != null && FTPClientConfig.SYST_UNIX_TRIM_LEADING.equals(config2.getServerSystemKey()))
                 });
         }
     }
