@@ -27,7 +27,7 @@ import org.apache.commons.net.ftp.FTPFile;
  * Implementation of FTPFileEntryParser and FTPFileListParser for IBM zOS/MVS
  * Systems.
  *
- * @version $Id: MVSFTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: MVSFTPEntryParser.java 1752660 2016-07-14 13:25:39Z sebb $
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for
  *      usage instructions)
  */
@@ -548,7 +548,7 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
     @Override
     protected FTPClientConfig getDefaultConfiguration() {
         return new FTPClientConfig(FTPClientConfig.SYST_MVS,
-                DEFAULT_DATE_FORMAT, null, null, null, null);
+                DEFAULT_DATE_FORMAT, null);
     }
 
 }
