@@ -24,7 +24,7 @@ import org.apache.commons.net.ftp.FTPFile;
 /**
  * Implementation FTPFileEntryParser and FTPFileListParser for pre MacOS-X Systems.
  *
- * @version $Id: MacOsPeterFTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: MacOsPeterFTPEntryParser.java 1752660 2016-07-14 13:25:39Z sebb $
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for usage instructions)
  * @since 3.1
  */
@@ -87,7 +87,7 @@ public class MacOsPeterFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
     /**
      * The default constructor for a UnixFTPEntryParser object.
      *
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -103,7 +103,7 @@ public class MacOsPeterFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      *
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -272,8 +272,7 @@ public class MacOsPeterFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
         return new FTPClientConfig(
                 FTPClientConfig.SYST_UNIX,
                 DEFAULT_DATE_FORMAT,
-                DEFAULT_RECENT_DATE_FORMAT,
-                null, null, null);
+                DEFAULT_RECENT_DATE_FORMAT);
     }
 
 }
