@@ -30,7 +30,7 @@ import org.apache.commons.net.ftp.FTPFile;
  * for more details.
  *
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for usage instructions)
- * @version $Id: NetwareFTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: NetwareFTPEntryParser.java 1752660 2016-07-14 13:25:39Z sebb $
  * @since 1.5
  */
 public class NetwareFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
@@ -58,7 +58,7 @@ public class NetwareFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
     /**
      * The default constructor for a NetwareFTPEntryParser object.
      *
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -73,7 +73,7 @@ public class NetwareFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
      *
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -171,8 +171,7 @@ public class NetwareFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
     @Override
     protected FTPClientConfig getDefaultConfiguration() {
         return new FTPClientConfig(FTPClientConfig.SYST_NETWARE,
-                DEFAULT_DATE_FORMAT, DEFAULT_RECENT_DATE_FORMAT, null, null,
-                null);
+                DEFAULT_DATE_FORMAT, DEFAULT_RECENT_DATE_FORMAT);
     }
 
 }
