@@ -24,7 +24,7 @@ import org.apache.commons.net.ftp.FTPFile;
 /**
  * Implementation of FTPFileEntryParser and FTPFileListParser for OS2 Systems.
  *
- * @version $Id: OS2FTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: OS2FTPEntryParser.java 1752660 2016-07-14 13:25:39Z sebb $
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for usage instructions)
  */
 public class OS2FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
@@ -46,7 +46,7 @@ public class OS2FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
     /**
      * The default constructor for a OS2FTPEntryParser object.
      *
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -62,7 +62,7 @@ public class OS2FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      *
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -140,7 +140,7 @@ public class OS2FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
         return new FTPClientConfig(
                 FTPClientConfig.SYST_OS2,
                 DEFAULT_DATE_FORMAT,
-                null, null, null, null);
+                null);
     }
 
 }
