@@ -24,7 +24,7 @@ import org.apache.commons.net.ftp.FTPClientConfig;
 import org.apache.commons.net.ftp.FTPFile;
 
 /**
- * @version $Id: OS400FTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: OS400FTPEntryParser.java 1752660 2016-07-14 13:25:39Z sebb $
  * <pre>
  * Example *FILE/*MEM FTP entries, when the current
  * working directory is a file of file system QSYS:
@@ -249,7 +249,7 @@ public class OS400FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
     /**
      * The default constructor for a OS400FTPEntryParser object.
      *
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -265,7 +265,7 @@ public class OS400FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      *
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -429,7 +429,7 @@ public class OS400FTPEntryParser extends ConfigurableFTPFileEntryParserImpl
         return new FTPClientConfig(
                 FTPClientConfig.SYST_OS400,
                 DEFAULT_DATE_FORMAT,
-                null, null, null, null);
+                null);
     }
 
 }
