@@ -30,7 +30,7 @@ import org.apache.commons.net.ftp.FTPFile;
  * This class is based on the logic of Daniel Savarese's
  * DefaultFTPListParser, but adapted to use regular expressions and to fit the
  * new FTPFileEntryParser interface.
- * @version $Id: UnixFTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: UnixFTPEntryParser.java 1781925 2017-02-06 16:43:40Z sebb $
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for usage instructions)
  */
 public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
@@ -45,6 +45,17 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
     static final String NUMERIC_DATE_FORMAT
         = "yyyy-MM-dd HH:mm"; //2001-11-09 20:06
 
+    // Suffixes used in Japanese listings after the numeric values
+    private static final String JA_MONTH = "\u6708";
+    private static final String JA_DAY   = "\u65e5";
+    private static final String JA_YEAR  = "\u5e74";
+
+    private static final String DEFAULT_DATE_FORMAT_JA
+        = "M'" + JA_MONTH + "' d'" + JA_DAY + "' yyyy'" + JA_YEAR + "'"; //6月 3日 2003年
+
+    private static final String DEFAULT_RECENT_DATE_FORMAT_JA
+        = "M'" + JA_MONTH + "' d'" + JA_DAY + "' HH:mm"; //8月 17日 20:10
+
     /**
      * Some Linux distributions are now shipping an FTP server which formats
      * file listing dates in an all-numeric format:
@@ -61,7 +72,7 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
         new FTPClientConfig(
                 FTPClientConfig.SYST_UNIX,
                 NUMERIC_DATE_FORMAT,
-                null, null, null, null);
+                null);
 
     /**
      * this is the regular expression used by this parser.
@@ -111,16 +122,24 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
          *   [d]d MMM
          *   N.B. use non-space for MMM to allow for languages such as German which use
          *   diacritics (e.g. umlaut) in some abbreviations.
+         *   Japanese uses numeric day and month with suffixes to distinguish them
+         *   [d]dXX [d]dZZ
         */
-        + "((?:\\d+[-/]\\d+[-/]\\d+)|(?:\\S{3}\\s+\\d{1,2})|(?:\\d{1,2}\\s+\\S{3}))"
+        + "("+
+            "(?:\\d+[-/]\\d+[-/]\\d+)" + // yyyy-mm-dd
+            "|(?:\\S{3}\\s+\\d{1,2})" +  // MMM [d]d
+            "|(?:\\d{1,2}\\s+\\S{3})" + // [d]d MMM
+            "|(?:\\d{1,2}" + JA_MONTH + "\\s+\\d{1,2}" + JA_DAY + ")"+
+           ")"
 
         + "\\s+" // separator
 
         /*
            year (for non-recent standard format) - yyyy
            or time (for numeric or recent standard format) [h]h:mm
+           or Japanese year - yyyyXX
         */
-        + "(\\d+(?::\\d+)?)" // (20)
+        + "((?:\\d+(?::\\d+)?)|(?:\\d{4}" + JA_YEAR + "))" // (20)
 
         + "\\s" // separator
 
@@ -129,12 +148,12 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
 
     // if true, leading spaces are trimmed from file names
     // this was the case for the original implementation
-    private final boolean trimLeadingSpaces;
+    final boolean trimLeadingSpaces; // package protected for access from test code
 
     /**
      * The default constructor for a UnixFTPEntryParser object.
      *
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -150,7 +169,7 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      *
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -168,7 +187,7 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
      * @param trimLeadingSpaces if {@code true}, trim leading spaces from file names
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -228,7 +247,14 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
 
             try
             {
-                file.setTimestamp(super.parseTimestamp(datestr));
+                if (group(19).contains(JA_MONTH)) { // special processing for Japanese format
+                    FTPTimestampParserImpl jaParser = new FTPTimestampParserImpl();
+                    jaParser.configure(new FTPClientConfig(
+                            FTPClientConfig.SYST_UNIX, DEFAULT_DATE_FORMAT_JA, DEFAULT_RECENT_DATE_FORMAT_JA));
+                    file.setTimestamp(jaParser.parseTimestamp(datestr));
+                } else {
+                    file.setTimestamp(super.parseTimestamp(datestr));
+                }
             }
             catch (ParseException e)
             {
@@ -347,8 +373,7 @@ public class UnixFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
         return new FTPClientConfig(
                 FTPClientConfig.SYST_UNIX,
                 DEFAULT_DATE_FORMAT,
-                DEFAULT_RECENT_DATE_FORMAT,
-                null, null, null);
+                DEFAULT_RECENT_DATE_FORMAT);
     }
 
 }
