@@ -38,7 +38,7 @@ import org.apache.commons.net.ftp.FTPFile;
  * </B>
  * <P>
  *
- * @version $Id: VMSFTPEntryParser.java 1741829 2016-05-01 00:24:44Z sebb $
+ * @version $Id: VMSFTPEntryParser.java 1752660 2016-07-14 13:25:39Z sebb $
  *
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for usage instructions)
  * @see org.apache.commons.net.ftp.parser.DefaultFTPFileEntryParserFactory
@@ -65,7 +65,7 @@ public class VMSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
     /**
      * Constructor for a VMSFTPEntryParser object.
      *
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -81,7 +81,7 @@ public class VMSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      *
      * @param config The {@link FTPClientConfig configuration} object used to
      * configure this parser.
-     * @exception IllegalArgumentException
+     * @throws IllegalArgumentException
      * Thrown if the regular expression is unparseable.  Should not be seen
      * under normal conditions.  It it is seen, this is a sign that
      * <code>REGEX</code> is  not a valid regular expression.
@@ -206,7 +206,7 @@ public class VMSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
      * read.
      *
      * @return A string representing the next ftp entry or null if none found.
-     * @exception IOException thrown on any IO Error reading from the reader.
+     * @throws IOException thrown on any IO Error reading from the reader.
      */
     @Override
     public String readNextEntry(BufferedReader reader) throws IOException
@@ -245,7 +245,7 @@ public class VMSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl
         return new FTPClientConfig(
                 FTPClientConfig.SYST_VMS,
                 DEFAULT_DATE_FORMAT,
-                null, null, null, null);
+                null);
     }
 
     // DEPRECATED METHODS - for API compatibility only - DO NOT USE
