@@ -30,6 +30,7 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 
 public class Threader {
 
@@ -108,7 +109,8 @@ public class Threader {
         // be a duplicate id, in which case we will need to generate a bogus placeholder id
         if (container != null) {
             if (container.threadable != null) { // oops! duplicate ids...
-                id = "<Bogus-id:" + (bogusIdCount++) + ">";
+                bogusIdCount++ ; // Avoid dead local store warning
+                id = "<Bogus-id:" + (bogusIdCount) + ">";
                 container = null;
             } else {
                 // The container just contained a forward reference to this message, so let's
@@ -211,11 +213,11 @@ public class Threader {
      */
     private ThreadContainer findRootSet(HashMap<String, ThreadContainer> idTable) {
         ThreadContainer root = new ThreadContainer();
-        Iterator<String> iter = idTable.keySet().iterator();
+        Iterator<Map.Entry<String, ThreadContainer>> iter = idTable.entrySet().iterator();
 
         while (iter.hasNext()) {
-            Object key = iter.next();
-            ThreadContainer c = idTable.get(key);
+            Map.Entry<String, ThreadContainer> entry = iter.next();
+            ThreadContainer c = entry.getValue();
             if (c.parent == null) {
                 if (c.next != null) {
                     throw new RuntimeException(
